var LOGFILE_NAME = "myenersave.log";
var l = new Logger(LOGFILE_NAME);

var numValues = 2;
var minResolution = 60;
var verbose = 0;

function addLeadingZeroes(number, count)
{
    var result = number + '';
    while (result.length < count) 
    {
        result = '0' + result;
    }
    return result;
}

function getNeededDSMs()
{
    var oReturnMeters=[]
    var Meters=getDSMeters();
    for (var index in Meters)
    {
        var oMeter = Meters[index];
        var oMeterDSID;
        try {
            oMeterDSID = Property.getNode("dSM/" + oMeter.dsid + "/dSID");
            if (oMeterDSID && Property.getProperty("dSM/" + oMeter.dsid + "/enabled")) {
                oReturnMeters.push(oMeter);
            } else {
                oReturnMeters.push(oMeter);
            }
        } catch(e) {
            oReturnMeters.push(oMeter);
        }
    }
    return oReturnMeters;
}

function sendMeterData()
{
    var apikey=Property.getProperty("apiKey");
    if  ((apikey==null) || (apikey==""))
    {
        return;
    }

    var Meters = getNeededDSMs();
    var meterDSID = [];
    var iMeterCounter = 0;
    var iValueCounter = 0;

    var content = '<upload>\r\n';
    for (var index in Meters)
    {
        var oMeter = Meters[index];
        var presentString="/apartment/dSMeters/" + oMeter.dsid +"/present"
        if (Property.getProperty(presentString)) {
            var oValues;
            try {
                oValues = Metering.getValues(oMeter.dsid, "energy",
                    minResolution, "Wh",
                    0, 0,
                    numValues + 2);
            } catch(e) {
                if (verbose >= 1) {
                    l.logln("Cannot request metering values from " + oMeter.dsid + ":");
                    for (var i in e) {
                        l.logln(i + ': ' + e[i]);
                    }
                }
                oValues = [];
            }
            iMeterCounter++;
            content += '  <sensor id="' + oMeter.dsid + '" type="10" description="' + oMeter.name + '">\r\n';

            for (var mIndex = 0; mIndex < oValues.length; mIndex++) {
                var sValue = oValues[mIndex];
                if (sValue != null) {
                    if (verbose >= 2) {
                        l.logln(" dSM ", oMeter.dsid, ", Wert: ", sValue.timestamp, " / ", sValue.value);
                    }
                    iValueCounter++;
                    var year = parseInt(sValue.timestamp.substring(0,4),10);
                    var mont = parseInt(sValue.timestamp.substring(5,7),10);
                    mont-=1;
                    var day = parseInt(sValue.timestamp.substring(8,10),10);
                    var hour = parseInt(sValue.timestamp.substring(11,13),10);
                    var minute = parseInt(sValue.timestamp.substring(14,16),10);
                    var second = parseInt(sValue.timestamp.substring(17,19),10);

                    var utz = new Date(year,mont,day,hour,minute,second,0);
                    var utzVal = Math.floor(utz.getTime() / 1000); //(utz.getTimezoneOffset()*60)+

                    content += '   <energy time="' + utzVal + '" reg="'+ sValue.value + '"/>\r\n';
                }
            }
            content += '  </sensor>\r\n';
        }
    }
    content += '</upload>\r\n';
    if (verbose >= 3) {
        print(content);
    }

    var sString = "Upload values from " + iMeterCounter + " active dSMeter(s)";

    var extraheaders = [];
    extraheaders['Host'] = 'data.myenersave.com:80';
    extraheaders['Connection'] = 'Close';
    extraheaders['Content-Type'] = 'application/xml';
    extraheaders['Content-Length'] = '' + content.length;
    extraheaders['Accept'] = 'text/html';
    extraheaders['User-Agent'] = 'dSS metering';
    extraheaders['Token'] = apikey;

    try {
        var h = new HTTP( { 'debug' : 0 }, extraheaders);
        var code = h.post('http://data.myenersave.com/fetcher/data', '/fetcher/data', content);
        if (code.status.valueOf() === 200) {
            l.logln(sString + ': OK [' + code.status + ']');
        } else {
            l.logln(sString + ': FAILED! Server replied: "' + code.status + '"');
            if (verbose >= 3) {
                print(code.body);
            }
        }
    } catch (e) {
        l.logln("Error during upload:");
        for (var i in e) {
            print(i + ': ' + e[i]);
        }
    }
}

function prepare()
{
    if (raisedEvent.parameter.apikey==null)
        return;
    var prop = new Property('apiKey');
    prop.setValue(raisedEvent.parameter.apikey);
    prop.setFlag('ARCHIVE', true);

    var Meters=getDSMeters();
    for (var index in Meters)
    {
        var oMeter=Meters[index];
        if (Property.getNode("dSM/" + oMeter.dsid + "/dSID")==null)
        {
            var prop = new Property("dSM/" + oMeter.dsid + "/dSID");
            prop.setValue(oMeter.dsid);
            prop.setFlag('ARCHIVE', true);
            prop = new Property("dSM/" + oMeter.dsid + "/enabled");
            prop.setValue(true);
            prop.setFlag('ARCHIVE', true);
        }
    }

    Property.store();
    sendMeterData();
}

function uploadCleanup()
{
    var id = Property.getProperty('id');
    var evtNode = Property.getNode('/system/EventInterpreter/ScheduledEvents');
    var propNode = evtNode.getChild(id);
    if (propNode) {
        l.logln("Cleanup old schedule " + id);
        propNode.getParent().removeChild(propNode);
    }
    Property.setProperty('id', 0);
}

function uploadScheduler()
{
    uploadCleanup();

    var ecount = Property.getProperty('count');
    Property.setProperty('count', ecount+1);

    var d = new Date();
    var diff = (numValues * minResolution) - d.getSeconds() + 20;

    var uploadEvent = new TimedEvent("myenersave", "+" + diff, { 'actions' : 'sendMeterData' });
    var id = uploadEvent.raise();
    Property.setProperty('id', id);

    if (verbose >= 1) {
        l.logln("Reschedule my id " + id + " in " + diff + "s");
    }
}

function main()
{
    if (raisedEvent.name === "running") {
        Property.load();
    }

    else if (raisedEvent.name === "myenersave") {
        if (raisedEvent.parameter.actions) {

            if (verbose >= 1) {
                l.logln('Execute: ' + raisedEvent.name + ' ' + raisedEvent.parameter.actions);
            }

            if (raisedEvent.parameter.actions == 'load') {
                Property.load();
            }
            if (raisedEvent.parameter.actions == 'save') {
                Property.store();
            }
            if (raisedEvent.parameter.actions == 'prepare') {
                prepare();
            }
            if (raisedEvent.parameter.actions == 'ready') {
                var prop = new Property('count');
                prop.setValue(0);
                uploadScheduler();
            }
            if (raisedEvent.parameter.actions == 'sendMeterData') {
                sendMeterData();
                uploadScheduler();
            }
        }
    }
}

main();
